using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Exchange_Component_Highlighting(WindowKey wk, Canvas canvas, ref View view, Model model, SegmentKey modelKey)
        {
            try
            {

            /* Sample: Exchange_Component_Highlighting
	        * This sample will show you how to:
	        * - Walk the component structure of a CAD Model
	        * - Highlight a specific instance of a component
	        * - Highlight all instances of a component
	        * - Build a KeyPath
	        * - Define a Named Style used for highlighting
	        * 
	        * Read more about the topics covered in this sample in our Programming Guide 
	        * - Integrating HPS with HOOPS Exchange
	        * - Section 6.2:		Highlighting
	        * - Section 1.3.1:		Using Keys and Objects
	        * */

	        /* This next block of code shows you how to load an Exchange file
	        * 1. Create an ImportOptionsKit
	        *		NOTE: It is required to import BRep data to access geometry referenced by PMI
	        * 2. Use the Exchange::File::Import method to load the desired file, with the options you specified
	        * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	        * 4. Use a try-catch block to capture any Exchange IO exceptions 
	        * 5. Obtain a CADModel object through the Exchange::ImportNotifier */
            HPS.CADModel cadModel = new HPS.CADModel();
            HPS.Exchange.ImportNotifier exchangeNotifier;
	        try {
		        string file_in = INPUT_FOLDER + "/LandingGear.prc";
                HPS.Exchange.ImportOptionsKit importOptionsKit = new HPS.Exchange.ImportOptionsKit();
		        importOptionsKit.SetBRepData(true);
                exchangeNotifier = HPS.Exchange.File.Import(file_in, importOptionsKit);
		        exchangeNotifier.Wait();

		        cadModel = exchangeNotifier.GetCADModel();
	        }

            catch (HPS.IOException)
            {
		        string error_message;
		        error_message = "Exchange Import Unsuccessful";
                modelKey.InsertText(new HPS.Point(0, 0, 0), error_message);
		        view.FitWorld();
		        view.Update();
	        }

	        /* This block of code sets up a named style which will be used for highlighting
	        * 1. Obtain a portfolio key. The portfolio used will need to be visible from the segment
	        *		using the name style defined in it.
	        * 2. Create a segment which will contain the attributes for highlighting 
	        * 3. Use the DefineNamedStyle method to create a named style in a Portfolio
	        *		To create a named style, provide it with a name, and specify the segment key
	        *		of the segment containing the attributes used for highlighting */
            HPS.PortfolioKey pKey;
	        view.GetSegmentKey().GetPortfolioControl().ShowTop(out pKey);

            HPS.SegmentKey redSegment = view.GetSegmentKey().Subsegment();
            redSegment.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(1, 0, 0, 1))
                .SetTextColor(new HPS.RGBAColor(1, 0.65f, 0, 1))
                .SetLineColor(new HPS.RGBAColor(1, 0.65f, 0, 1));
	        pKey.DefineNamedStyle("redHighlight", redSegment);

	        /* This block of code sets up our scene:
	            * We will have two subwindows, each including an instance of the CAD Model we have imported:
	            * 1. Activate the default capture from the CAD Model
	            * 2. Include the default capture into two segments, these two segments will house the two subwindows */
            HPS.SegmentKey view1, view2;
	        view1 = view.GetSegmentKey().Subsegment();
	        view2 = view.GetSegmentKey().Subsegment();
            HPS.View defaultView = cadModel.ActivateDefaultCapture();
            HPS.IncludeKey view1Include = view1.IncludeSegment(defaultView.GetSegmentKey());
            HPS.IncludeKey view2Include = view2.IncludeSegment(defaultView.GetSegmentKey());

	        view1.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, -1, 1)).
                SetBorder(HPS.Subwindow.Border.Inset);
            view1.InsertDistantLight(new HPS.Vector(1, 1, 0));
	        view2.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, -1, 1)).
                SetBorder(HPS.Subwindow.Border.Inset);
            view2.InsertDistantLight(new HPS.Vector(1, 1, 0));
	
	        view.FitWorld();
	        view.Update();

	        /* This code shows how to walk the component structure of the CAD Model
	        * For this particular model we assume that we know where the data we wish to highlight is located
	        * The model loaded in this file represents the landing gear of an aeroplane.
	        * The landing gear contains four wheels. 
	        * Only one wheel prototype exists in the model, and four instances of it are then included
	        * where appropriate to represent four wheels without replicating data.
	        * 
	        * A key path to the specific part will be needed, and can be obtained using the GetKeyPath method */
            HPS.Component[] rootComponent = cadModel.GetSubcomponents();
            HPS.Component[] parts = rootComponent[0].GetSubcomponents();
            HPS.Component[] wheels = parts[2].GetSubcomponents();
            HPS.Component[] wheelInstance = wheels[1].GetSubcomponents();

            HPS.KeyPath[] wheelKeyPath;
            wheelKeyPath = HPS.Component.GetKeyPath(wheelInstance[0]);

	        /* This next block of code performs highlighting 
	            * The KeyPathArray object obtained above will contain four KeyPath objects:
	            * each one describing the path to one of the four instances of a wheel.
	            * 
	            * To highlight all instances of a wheel we complete the path all the way to
	            * the window for all KeyPath objects, and highlight them */
            HPS.HighlightOptionsKit highlightOptions = new HPS.HighlightOptionsKit();
	        highlightOptions.SetStyleName("redHighlight");

            foreach (HPS.KeyPath kp in wheelKeyPath)
	        {
                HPS.Key[] allInstanceskeys;
		        kp.ShowKeys(out allInstanceskeys);
                List<Key> allInstanceskeysList = new List<Key>(allInstanceskeys);
		        allInstanceskeysList.Add(view1Include);
		        allInstanceskeysList.Add(canvas.GetAttachedLayout().GetAttachedViewIncludeLink(0));
		        allInstanceskeysList.Add(canvas.GetAttachedLayoutIncludeLink());
		        allInstanceskeysList.Add(wk);

                HPS.KeyPath allInstanceshighlightKeyPath = new HPS.KeyPath(allInstanceskeysList.ToArray());
		        wk.GetHighlightControl().Highlight(allInstanceshighlightKeyPath, highlightOptions);
	        }

	        /* This code shows how to highlight only one instance of a component
	        * To highlight only one component we repeat the actions performed above, but this
	        * time, they are performed only on the specific KeyPath of interest. */
            HPS.KeyPath oneWheelPath = wheelKeyPath[0];
            HPS.Key[] keys;
	        oneWheelPath.ShowKeys(out keys);
            List<Key> keyList = new List<Key>(keys);
	        keyList.Add(view2Include);
	        keyList.Add(canvas.GetAttachedLayout().GetAttachedViewIncludeLink(0));
	        keyList.Add(canvas.GetAttachedLayoutIncludeLink());
	        keyList.Add(wk);

            HPS.KeyPath highlightKeyPath = new HPS.KeyPath(keyList.ToArray());
	        wk.GetHighlightControl().Highlight(highlightKeyPath, highlightOptions);

	        /* The next block adds text description to both subwindows. For each subwindow the following
	            * operations are performed:
	            * 1. A subsegment is created, and depth range is set on it to make sure that the geometry in this
	            *		segment will always appear on top
	            * 2. Text visibility is enabled through the VisibilityControl
	            * 3. A stretched camera is added to the subsegment
	            * 4. The text attributes are modified as needed using the TextAttributeControl
	            * 5. Text is inserted */
            HPS.SegmentKey view1Description = view1.Subsegment();
	        view1Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
	        view1Description.GetVisibilityControl().SetEverything(false).SetText(true);
            view1Description.GetCameraControl().SetPosition(new HPS.Point(0, 0, -5)).
                SetTarget(new HPS.Point(0, 0, 0)).
                SetUpVector(new HPS.Vector(0, 1, 0)).
				SetField(2, 2).
                SetProjection(HPS.Camera.Projection.Stretched);
            view1Description.GetTextAttributeControl().SetSize(0.02f, HPS.Text.SizeUnits.SubscreenRelative).
                SetAlignment(HPS.Text.Alignment.Center, HPS.Text.ReferenceFrame.WorldAligned, HPS.Text.Justification.CenterJustified);
            view1Description.InsertText(new HPS.Point(0, 0.85f, 0), "All instances of the component\n are highlighted in red");


            HPS.SegmentKey view2Description = view2.Subsegment();
	        view2Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
	        view2Description.GetVisibilityControl().SetEverything(false).SetText(true);
            view2Description.GetCameraControl().SetPosition(new HPS.Point(0, 0, -5)).
                SetTarget(new HPS.Point(0, 0, 0)).
                SetUpVector(new HPS.Vector(0, 1, 0)).
		        SetField(2, 2).
                SetProjection(HPS.Camera.Projection.Stretched);
	        view2Description.GetTextAttributeControl().SetSize(0.02f, Text.SizeUnits.SubscreenRelative).
                SetAlignment(HPS.Text.Alignment.Center, HPS.Text.ReferenceFrame.WorldAligned, HPS.Text.Justification.CenterJustified);
            view2Description.InsertText(new HPS.Point(0, 0.85f, 0), "Component instance is highlighted in red");

            }
            catch (IOException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}
